unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, MySQL4, StdCtrls;

type
  TInsertThread = class(TThread)
    procedure Execute; override;
  end;

  TDeleteThread = class(TThread)
    procedure Execute; override;
  end;

  TForm1 = class(TForm)
    ButtonStart: TButton;
    ButtonStop: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonStartClick(Sender: TObject);
    procedure ButtonStopClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    FStarted: Boolean;
    FMySQL: PMySQL;
    InsertThread: TInsertThread;
    DeleteThread: TDeleteThread;
  public
    { Public declarations }
    procedure InizioMainThread;
    procedure FineMainThread;
    procedure InizioInsertThread;
    procedure FineInsertThread;
    procedure InizioDeleteThread;
    procedure FineDeleteThread;
  end;


var
  Form1: TForm1;

implementation

{$R *.dfm}

function MySQLEncodeString(const MySQL: PMySql; const Source: WideString): String;
var
  Encoded, Escaped: String;
  Len: Integer;
begin
  Result := '';
  if Source = '' then
    exit;

  Len := WideCharToMultiByte(CP_UTF8, 0, PWideChar(Source), -1, nil, 0, nil, nil);
  if Len > 0 then begin
    SetLength(Encoded, Len - 1);
    WideCharToMultiByte(CP_UTF8, 0, PWideChar(Source), -1, PChar(Encoded), Len, nil, nil);
    SetLength(Escaped, Length(Encoded) * 2);
    Len := mysql_real_escape_string(MySQL, PChar(Escaped), PChar(Encoded), Length(Encoded));
    SetLength(Escaped, Len);
    Result := Escaped;
  end;
end;

//----------------------------------------------------------------------------------------
// Procedure  MySQLDecodeString
// Arguments  const Source: String
// Created on 27 agosto 2003 - 10:41:54
// Updated on 27 agosto 2003 by Cristiano Muzi <muzi@osway.com>
{:
   @author    Cristiano Muzi <muzi@osway.com>
   @result    WideString
   @raises    ExcName ExcDescription
   @summary   SummaryText
   @desc      This function converts a UTF-8 encoded multibyte string into a WideString.
   @bug       BugText
   @example   ExampleText
}
//----------------------------------------------------------------------------------------
function MySQLDecodeString(const Source: String): WideString;
var
  Decoded: WideString;
  Len: Integer;
begin
  Result := '';
  if Source = '' then
    exit;
  Len := MultiByteToWideChar(CP_UTF8, 0, PChar(Source), -1, nil, 0);
  if Len > 0 then begin
    SetLength(Decoded, Len - 1);
    MultiByteToWideChar(CP_UTF8, 0, PChar(Source), -1, PWideChar(Decoded), Len);
    Result := Decoded;
  end;
end;



function MySqlGetFieldValue(const Row: PMySql_Row; const Lengths: PMySql_Lengths; const Index: Integer): WideString;
var
  Field: String;
begin
  SetLength(Field, Lengths[Index]);
  if Lengths[Index] > 0 then
    Move(Row[Index]^, Field[1], Lengths[Index]);
  Result := MySqlDecodeString(Field);
end;


procedure CreateFilePreviewEntity(MySQL: PMySQL);
var Query: string;
begin
// NB: If the entity is created defining one or more indices, an error of kind "duplicate entry 'X' for key Y"
// will be raised.

  Query := 'CREATE TABLE IF NOT EXISTS FilePreview (' +
      'fileID INTEGER NOT NULL AUTO_INCREMENT,  ' +
      'filePath TEXT NOT NULL, ' +
      'testField INTEGER, ' +
      'PRIMARY KEY (fileID), ' +
      'INDEX testFieldIndex (testField), ' +
      'INDEX filePath (filePath(255)))';
(*
  Query := 'CREATE TABLE IF NOT EXISTS FilePreview (' +
      'fileID INTEGER NOT NULL AUTO_INCREMENT,  ' +
      'filePath TEXT NOT NULL, ' +
      'testField INTEGER, ' +
      'PRIMARY KEY (fileID))';
*)      

  if mysql_query(MySQL, PChar(Query)) <> 0 then
    raise Exception.Create('Error creating the entity: ' + mysql_error(MySql));
end;


procedure TForm1.FormCreate(Sender: TObject);
var
  MySQL: PMySQL;
begin
  if mysql_server_init(0, nil, nil) <> 0 then
    raise Exception.Create('mysql_server_init error');

  FMySQL := mysql_init(nil);
  if FMySQL = nil then
    raise Exception.Create('mysql_init error');

  mysql_options(FMySQL, MYSQL_READ_DEFAULT_GROUP, 'mysql_test_program');

  MySQL := mysql_real_connect(FMySQL, nil, nil , nil, nil, 0, nil, 0);
  if MySQL <> nil then
    try
      if mysql_query(MySQL, PChar('CREATE DATABASE IF NOT EXISTS foo')) <> 0 then
        raise Exception.Create('Cannot create the database.');
    finally
      mysql_close(FMySQL);
      mysql_thread_end;
    end
  else
    raise Exception.Create('mysql_real_connect error');

  MySQL := mysql_connect(nil, nil, nil, nil);
  mysql_select_db(MySQL, PChar('foo'));
  if MySQL <> nil then
    try
      CreateFilePreviewEntity(MySQL);
    finally
      mysql_close(MySQL);
      mysql_thread_end;
    end
  else
    raise Exception.Create('mysql_connect error');

  InsertThread := TInsertThread.Create(true);
  DeleteThread := TDeleteThread.Create(true);
end;


procedure TForm1.FormDestroy(Sender: TObject);
begin
  DeleteThread.Free;
  InsertThread.Free;
  mysql_server_end;
end;


procedure TForm1.ButtonStartClick(Sender: TObject);
var
  MySQL: PMySQL;
  QResult: PMySql_Res;
  QRow: PMySql_Row;
  QLengths: PMySql_Lengths;
  FileID, TestField: Integer;
  FilePath: string;
begin
  FStarted := true;

  InsertThread.Resume;
  DeleteThread.Resume;

  InizioMainThread;
  try
    while FStarted do begin
      MySQL := mysql_connect(nil, nil, nil, nil);
      mysql_select_db(MySQL, PChar('foo'));
      if MySQL <> nil then
        try
          if mysql_query(MySql, PChar('UPDATE FilePreview SET testField = 1 WHERE testField = 0')) <> 0 then
            raise Exception.Create('mysql_query error: ' + mysql_error(MySql));
          Application.ProcessMessages;
        finally
          mysql_close(MySQL);
          mysql_thread_end;
        end;
    end;
  finally
    FineMainThread;
  end;
end;

{ TInsertThread }

procedure TInsertThread.Execute;
var
  I: integer;
  MySQL: PMySQL;
begin
  I := 0;
  Synchronize(Form1.InizioInsertThread);
  try
    while (true) do begin
      if Terminated then
        break;
      MySQL := mysql_connect(nil, nil, nil, nil);
      mysql_select_db(MySQL, PChar('foo'));
      if MySQL <> nil then
        try
          if mysql_query(MySQL, PChar(Format('INSERT INTO FilePreview (filePath, testField) VALUES ("%s", %d)', [MySQLEncodeString(MySQL, 'foo'+IntToStr(I)), 0]))) <> 0 then
            raise Exception.Create('INSERT error - ' + mysql_error(MySql));
          Inc(I);
        finally
          mysql_close(MySQL);
          mysql_thread_end;
        end
      else
        raise Exception.Create('mysql_real_connect error');
    end;
  finally
    Synchronize(Form1.FineInsertThread);
  end;

end;

{ TDeleteThread }

procedure TDeleteThread.Execute;
var
  I: Integer;
  MySQL: PMySQL;
begin
  Synchronize(Form1.InizioDeleteThread);
  try
    while (true) do begin
      if Terminated then
        break;

      MySQL := mysql_connect(nil, nil, nil, nil);
      mysql_select_db(MySQL, PChar('foo'));
      if MySQL <> nil then
        try
          if mysql_query(MySQL, PChar(Format('DELETE FROM FilePreview WHERE testField = 1', []))) <> 0 then
            raise Exception.Create('DELETE error - ' + mysql_error(MySql));
          Inc(I);
        finally
          mysql_close(MySQL);
          mysql_thread_end;
        end
      else
        raise Exception.Create('mysql_real_connect error');
    end;
  finally
    Synchronize(Form1.FineDeleteThread);
  end;
end;

procedure TForm1.ButtonStopClick(Sender: TObject);
begin
  FStarted := false;
  DeleteThread.Terminate;
  InsertThread.Terminate;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  ButtonStopClick(Sender);
  CanClose := true;
end;

procedure TForm1.FineDeleteThread;
begin
  Label3.Color := clBlack;
  Label3.Invalidate;
end;

procedure TForm1.FineInsertThread;
begin
  Label2.Color := clBlack;
  Label2.Invalidate;
end;

procedure TForm1.FineMainThread;
begin
  Label1.Color := clBlack;
  Label1.Invalidate;
end;

procedure TForm1.InizioDeleteThread;
begin
  Label3.Color := clRed;
  Label3.Invalidate;
end;

procedure TForm1.InizioInsertThread;
begin
  Label2.Color := clRed;
  Label2.Invalidate;
end;

procedure TForm1.InizioMainThread;
begin
  Label1.Color := clRed;
  Label1.Invalidate;
end;

end.
